; ****************************
; ***      DOS Wedge       ***
; ***  21-Sep-2024 by KPK  ***
; ***   Assembler: Acme    ***
; ****************************

* = $c000     ; start address (49152)

init:         ; set up wedge routine
  jsr show_help ; list available commands
  lda #<wedge
  cmp $308
  bne +
  lda #>wedge
  cmp $309
  bne +
  beq init_end ; skip if already loaded
+ lda $308
  sta bas_vector
  lda $309
  sta bas_vector+1
  lda #<wedge
  sta $308
  lda #>wedge
  sta $309
init_end:
  rts

; --------------------------------------------------------------------

wedge:        ; memory-resident dos wedge routine
  ldy #1
  lda ($7a),y ; "look ahead" at the instruction
  cmp #'@'    ; wedge command to get drive status
  beq at_cmd  ; process "at" (@) or help command
  cmp #'$'    ; wedge command to get directory
  beq get_dir
  cmp #$5c    ; wedge command to get start address ('£')
  beq start_addr ; get starting address of file
  cmp #$ad    ; basic token for '/' character
  beq loadbas ; load basic file
  cmp #'%'    ; wedge command to load ml file
  beq loadml  ; load ml file
  cmp #$ae    ; basic token for up arrow character
  beq run     ; load file, then run
  jmp (bas_vector) ; no wedge command selected, so continue

; --------------------------------------------------------------------

at_cmd:       ; "at" (@) command
  iny
  lda ($7a),y ; "look ahead" at the instruction
  cmp #$99    ; basic token for '?' (print)
  bne +
  jsr show_help
at_done:
- jsr $73     ; get next character
  bne -       ; until there are no more to get
  jmp $a7ae   ; next statement for execution (NEWSTT)
+ cmp #'U'    ; command to uninstall dos wedge
  bne drv_status
  jsr uninstall_wedge
  jmp at_done

drv_status:   ; get drive status
  jsr $73
  jsr drive_status
  jmp (bas_vector)

get_dir:      ; get directory
  jsr parse_file_name
  jsr get_directory
  jmp $a7ae   ; next statement for execution (NEWSTT)

start_addr:   ; get starting address of file
  jsr start_address
  jmp $a7ae   ; next statement for execution (NEWSTT)

loadbas:      ; load basic file
  ldx #0
  stx secondary
lbmain:
  stx file_error
  jsr $73
  jsr load_file
  lda file_error
  beq +       ; no errors, so continue
  jsr print_file_error
+ jmp $a7ae   ; next statement for execution (NEWSTT)

loadml:       ; load ml file
  ldx #1
  stx secondary
  ldx #0
  jmp lbmain

run:          ; load file, then run
  jsr $73
  ldx #0
  stx file_error
  jsr load_file
  lda file_error
  beq +       ; no errors, so continue
  jsr print_file_error
  jmp rdone
+ ldx #0
- lda run_cmd,x
  beq +
  sta $277,x  ; stuff keyboard buffer with command
  inx
  bne -
+ stx 198     ; length of command string
rdone:
  jmp $a7ae   ; next statement for execution (NEWSTT)

; --------------------------------------------------------------------

show_help:    ; list available commands
  lda #<help_msg
  ldy #>help_msg
  jsr $ab1e   ; print string (STROUT)
  rts

uninstall_wedge: ; uninstall dos wedge
  lda bas_vector
  sta $308
  lda bas_vector+1
  sta $309
  lda #<uninst_msg ; display wedge uninstalled message
  ldy #>uninst_msg
  jsr $ab1e   ; print string (STROUT)
  rts

drive_status: ; get drive status
  lda #0      ; no filename
  ldx #0
  ldy #0
  jsr $ffbd   ; call SETNAM
  lda #15     ; file number 15 (command channel)
  ldx $ba     ; default device
  bne +
  ldx #8      ; set device number to 8
+ ldy #15     ; secondary address 15 (error channel)
  jsr $ffba   ; call SETLFS
  jsr $ffc0   ; call OPEN
  bcs ds_end
  ldx #15     ; file number 15
  jsr $ffc6   ; call CHKIN (file 15 now used as input)
- jsr $ffb7   ; call READST (read status byte)
  bne ds_end  ; either EOF or read error
  jsr $ffcf   ; call CHRIN (get a byte from file)
  jsr $ffd2   ; call CHROUT (print byte to screen)
  bne -       ; next byte
ds_end:
  lda #15     ; file number 15
  jsr $ffc3   ; call close
  jsr $ffcc   ; call CLRCHN
  rts

; --------------------------------------------------------------------

get_directory:
  ldx #<str_name
  ldy #>str_name
  lda str_len
  jsr $ffbd   ; set filename
  lda #$08
  sta $ba     ; device #8
  lda #$60
  sta $b9     ; secondary chn
  jsr $f3d5   ; open for serial bus devices
  jsr $f219   ; set input device
  ldy #$04
gd_loop:
- jsr $ee13   ; input byte on serial bus
  dey
  bne -       ; get rid of y bytes
  lda $c6     ; key pressed?
  ora $90     ; or eof?
  bne +       ; if yes exit
  jsr $ee13   ; get block size in high/low bytes (ax)
  tax
  jsr $ee13
  jsr $bdcd   ; print number from ax
  lda #' '
  jsr $ffd2   ; print a space for separation
- jsr $ee13   ; now the filename
  jsr $e716   ; put a character to screen
  bne -       ; while 0 not encountered
  jsr $aad7   ; print cr at the end of the line
  ldy #$02    ; set 2 bytes to skip
  bne gd_loop ; repeat
+ jsr $f642   ; close serial bus device
  jsr $f6f3   ; restore i/o devices to default
  rts

; --------------------------------------------------------------------

start_address:
  jsr parse_file_name
  ldx #<str_name
  ldy #>str_name
  lda str_len
  jsr $ffbd   ; call SETNAM
  ldx $ba     ; last used device number
  bne +
  ldx #$08    ; default to device 8 if none is selected
+ ldy #$02    ; secondary address 2
  lda #$01    ; logical file number
  jsr $ffba   ; call SETLFS
  jsr $ffc0   ; call OPEN
  bcs sa_done
  ldx #1      ; file number 1
  jsr $ffc6   ; call CHKIN (file 1 now used as input)
  jsr $ffb7   ; call READST (read status byte)
  bne sa_done ; either EOF or read error
  jsr $ffcf   ; call CHRIN (get a byte from file)
  sta start_add ; write byte to memory
  jsr $ffb7   ; call READST (read status byte)
  bne sa_done ; either EOF or read error
  jsr $ffcf   ; call CHRIN (get a byte from file)
  sta start_add+1 ; write byte to memory
  ldx start_add
  lda start_add+1
  jsr $bdcd   ; print 16-bit number (a = hi, x = lo)
sa_done:
  lda #$01    ; file number 1
  jsr $ffc3   ; call CLOSE
  jsr $ffcc   ; call CLRCHN
  rts

; --------------------------------------------------------------------

load_file:
  jsr parse_file_name
  ldx #<str_name
  ldy #>str_name
  lda str_len
  jsr $ffbd   ; call SETNAM
  ldx $ba     ; last used device number
  bne +
  ldx #$08    ; default to device 8 if none is selected
+ ldy secondary ; secondary address
  lda #$01    ; logical file number
  jsr $ffba   ; call SETLFS
  ldx #$01    ; Load address ($801 = start of Basic)
  ldy #$08
  lda #$00    ; $00 means: load to memory (not verify)
  jsr $ffd5   ; call LOAD
  bcc +
  sta file_error
+ stx $2d     ; start of basic variable storage (VARTAB)
  sty $2e
  rts

; --------------------------------------------------------------------

print_file_error:
  ldx file_error ; error occurred, so print...
  jsr $e38b   ; relevant error message
  rts

; --------------------------------------------------------------------

parse_file_name:
  ; check whether there are any quotes in filename
  ldy #1
  ldx #0      ; use x as a flag for quote in filename
- lda ($7a),y
  beq ++
  cmp #$22    ; quote symbol
  bne +
  inx
+ iny
  bne -
++ cpx #0
  bne with_quotes

  ; parse without quotes in filename
  ldy #1
  ldx #0
- lda ($7a),y
  beq pdone
  cmp #$ac    ; token for '*'
  bne +
  lda #'*'
+ cmp #$99    ; token for '?'
  bne +
  lda #'?'
+ cmp #' '    ; ignore spaces
  bne +
  iny
  bne -
+ sta str_name,x
  inx
  iny
  bne -
  jmp pdone

with_quotes:  ; parse with quotes in filename
  ldx #0
  ldy #1
  sty buf_ignore
- lda ($7a),y
  beq pdone
  cmp #$ac    ; token for '*'
  bne +
  lda #'*'
+ cmp #$99    ; token for '?'
  bne +
  lda #'?'
+ cmp #$22    ; check for quote
  bne +
  lda #1
  eor buf_ignore
  sta buf_ignore
  bne pdone   ; if closing quote, we are done
  ldx #0      ; if opening quote, start over
  beq pnext
+ sta str_name,x
  inx
pnext:
  iny
  bne -

pdone:
  stx str_len
- jsr $73     ; get next character
  bne -       ; until there are no more to get
  rts

; --------------------------------------------------------------------

bas_vector: !fill 2 ; basic vector to IGONE
start_add:  !fill 2 ; start address of file
file_error: !fill 1 ; file error number
buf_ignore: !fill 1 ; flag to ignore byte in input buffer
secondary:  !fill 1 ; secondary address (0 = Basic, 1 = ML)
str_len:    !fill 1 ; length of file name string
run_cmd:    !pet "run",13,0 ; string used to fill keyboard buffer
help_msg:   !pet 13,"*** dos wedge - 21-sep-2024 by kpk ***",13,13
            !pet "available commands:",13
            !pet "$  = directory",13
            !pet "/  = load basic file",13
            !pet "%  = load machine language file",13
            !pet $5e,"  = load and run file",13
            !pet $5c,"  = starting address of file",13
            !pet "@  = drive status",13
            !pet "@? = this help",13
            !pet "@u = uninstall",13,0
uninst_msg: !pet 13,"dos wedge uninstalled",13,0
str_name:   ; file name string - generated at run time